@@ -497,6 +497,31 @@ parsednssl (const uint8_t *opt)
 }
 
 
+static int
+parsepref64 (const uint8_t *opt)
+{
+	static const uint8_t preflen[] = { 96, 64, 56, 48, 40, 32 };
+	uint16_t lifetime_plc = ntohs(((const uint16_t *)opt)[1]);
+	unsigned lifetime = lifetime_plc & 0xfff8;
+	unsigned plc = lifetime_plc & 0x0007;
+	struct in6_addr pref64;
+	char str[INET6_ADDRSTRLEN];
+
+	if (opt[1] != 2 || plc > 5)
+		return -1;
+
+	memcpy(&pref64, opt + 4, 12);
+	if (inet_ntop (AF_INET6, &pref64, str, sizeof (str)) == NULL)
+		return -1;
+
+	printf (_(" NAT64 prefix             : %s/%"PRIu8"\n"), str,
+		preflen[plc]);
+	printf (_("  NAT64 prefix lifetime   : %12u (    0x%04x) %s\n"),
+		lifetime, lifetime, ngettext ("second", "seconds", lifetime));
+	return 0;
+}
+
+
 static int
 parseadv (const uint8_t *buf, size_t len, const struct sockaddr_in6 *tgt,
           bool verbose)
@@ -613,6 +638,10 @@ parseadv (const uint8_t *buf, size_t len, const struct sockaddr_in6 *tgt,
 			case 31: // RFC6106
 				parsednssl (ptr);
 				break;
+
+			case 38: // RFC8781
+				parsepref64 (ptr);
+				break;
 		}
 		/* skips unrecognized option */
 
@@ -731,11 +760,6 @@ recvadv (int fd, const struct sockaddr_in6 *tgt, unsigned wait_ms,
 		 && (addr.sin6_scope_id != tgt->sin6_scope_id))
 			continue;
 
-		/* We don't want non-matching addresses for unicast targets */
-		if (!IN6_IS_ADDR_MULTICAST(&tgt->sin6_addr)
-		 && memcmp(&addr.sin6_addr, &tgt->sin6_addr, 16))
-			continue;
-
 		if (parseadv (buf.b, val, tgt, (flags & NDISC_VERBOSE) != 0) == 0)
 		{
 			if (flags & NDISC_VERBOSE)
